from .imports import *
from .directory_utils import *
def get_file_name(file_path: str) -> str:
    """
    Retrieves and returns the base name of a file from a given file path.

    Args:
        file_path (str): A string representing the file path.

    Returns:
        str: The base name of the file (without extension).
    """
    return split_text(get_base_name(file_path))[0]
def get_abs_name_of_this():
    """
    Returns the absolute name of the current module.

    Returns:
        Path: The absolute name of the current module.
    """
    return os.path.abspath(__name__)
def sanitize_filename(name: str):
    """
    Sanitize a filename by removing invalid characters.
    
    Args:
    name (str): Filename to sanitize.
    
    Returns:
    str: Sanitized filename.
    """
    return re.sub(r'[\\/*?:"<>|]', "", name)
def get_base_name(file_path: str) -> str:
    """
    Extracts and returns the base name of a file from a given file path.

    Args:
        file_path (str): A string representing the file path.

    Returns:
        str: The base name of the file.
    """
    return os.path.basename(file_path)



import ast
from pathlib import Path
from importlib.util import find_spec
from typing import Dict, Set
import ast, sys
from pathlib import Path
from importlib.util import find_spec
from typing import Dict, Set
from src.abstract_utilities.import_utils import *
STDLIB_NAMES = set(sys.builtin_module_names)
def parse_imports(file_path: Path):
    """Return list of (module, level) for every import/from-import."""
    try:
        src = file_path.read_text(errors="ignore")
        tree = ast.parse(src, filename=str(file_path))
    except Exception:
        return []
    imports = []
    for node in ast.walk(tree):
        if isinstance(node, ast.Import):
            for alias in node.names:
                imports.append((alias.name, 0))
        elif isinstance(node, ast.ImportFrom):
            imports.append((node.module, node.level))
    return imports


def resolve_relative_import(base_file: Path, module: str | None, level: int) -> Path | None:
    """Follow a relative import path to its real file if it exists."""
    base = base_file.parent
    for _ in range(level - 1):
        base = base.parent
    if not module:
        target = base
    else:
        target = base / module.replace(".", "/")
    if (target / "__init__.py").exists():
        return target / "__init__.py"
    if target.with_suffix(".py").exists():
        return target.with_suffix(".py")
    return None




def classify_import(mod_name: str, root_pkg: str) -> str:
    """Return 'local', 'internal', or 'external'."""
    if not mod_name:
        return "unknown"
    if mod_name.startswith("."):
        return "local"
    if mod_name.split(".")[0] == root_pkg:
        return "internal"
    if mod_name.split(".")[0] in STDLIB_NAMES:
        return "stdlib"
    return "external"


def follow_imports(entry: Path, root_pkg: str,
                   visited: Dict[Path, Dict[str, Set[str]]] | None = None):
    """
    Recursively follow only internal/local imports.
    Returns {file_path: {'internal': set(), 'external': set()}}
    """
    visited = visited or {}
    if entry in visited:
        return visited

    visited[entry] = {"internal": set(), "external": set()}

    for mod, level in parse_imports(entry):
        if level > 0:
            target = resolve_relative_import(entry, mod, level)
            if target:
                visited[entry]["internal"].add(str(target))
                follow_imports(target, root_pkg, visited)
            continue

        category = classify_import(mod, root_pkg)
        if category == "internal":
            spec = find_spec(mod)
            if spec and spec.origin and spec.origin.endswith(".py"):
                visited[entry]["internal"].add(spec.origin)
                follow_imports(Path(spec.origin), root_pkg, visited)
        elif category == "external":
            visited[entry]["external"].add(mod)
        elif category == "stdlib":
            # stdlib gets treated like external but labeled
            visited[entry]["external"].add(mod + "  # stdlib")

    return visited



def build_master_imports(entry: Path, root_pkg: str, output: Path):
    trace = follow_imports(entry, root_pkg)
    lines = ["# Auto-generated master imports for abstract_utilities\n"]
    all_modules = set()
    external_modules = set()
    imports = get_all_imports(path)
    for _, data in trace.items():
        for dep in data["internal"]:
            path = Path(dep)
            if path.suffix != ".py":
                continue
            try:
                rel_parts = path.with_suffix("").parts
                idx = rel_parts.index(root_pkg)
                dotted = ".".join(rel_parts[idx:])
                all_modules.add(dotted)
            except ValueError:
                continue
        external_modules.update(data["external"])

    for mod in sorted(all_modules):
        short = mod.split(".", 1)[-1]
        lines.append(f"from .{short} import *")

    if external_modules:
        lines.append("\n# External / stdlib imports (not traced, for reference)")
        for ext in sorted(external_modules):
            lines.append(f"# {ext}")
    
    output.parent.mkdir(parents=True, exist_ok=True)
    output.write_text("\n".join(lines+str(imports)))
    print(f"✅ wrote master imports hub → {output}")



if __name__ == "__main__":
    entry = Path(
        "/home/flerb/Documents/pythonTools/modules/src/modules/abstract_utilities/src/"
        "abstract_utilities/import_utils/src/import_functions.py"
    )

    
    pkg = "abstract_utilities"
    out = entry.parents[4] / "imports" / "__init__.py"
    build_master_imports(entry, pkg, out)



from abstract_utilities import *
import os
from collections import defaultdict

def get_path_or_init(pkg_info):
    root_dirname = pkg_info.get("root_dirname")
    pkg = pkg_info.get("pkg")
    rel_path = pkg.replace('.', '/')
    dirname = os.path.dirname(root_dirname)
    pkg_path = os.path.join(dirname, rel_path)
    pkg_py_path = f"{pkg_path}.py"
    if os.path.isfile(pkg_py_path):
        return pkg_py_path
    pkg_init_path = os.path.join(pkg_path, '__init__.py')
    if os.path.isdir(pkg_path) and os.path.isfile(pkg_init_path):
        return pkg_init_path
    # optional: silence instead of blocking input()
    print(f"⚠️ not found == {pkg_info}")
    return None


def get_dot_fro_line(line, dirname=None, file_path=None, get_info=False):
    info_js = {"nuline": line, "og_line": line, "pkg": line, "dirname": dirname,
               "file_path": file_path, "root_dirname": None, "local": False}
    if dirname and is_file(dirname):
        file_path = dirname
        dirname = os.path.dirname(dirname)
        info_js["file_path"] = file_path
        info_js["dirname"] = dirname

    from_line = line.split(FROM_TAG)[-1]
    dot_fro = ""
    for char in from_line:
        if char != '.':
            pkg = f"{dot_fro}{eatAll(from_line, '.')}"
            nuline = f"from {pkg}"
            info_js["nuline"] = nuline
            info_js["pkg"] = pkg
            break
        if dirname:
            info_js["root_dirname"] = dirname
            dirbase = os.path.basename(dirname)
            dirname = os.path.dirname(dirname)
            dot_fro = f"{dirbase}.{dot_fro}"

    if get_info:
        if dot_fro and os.path.isdir(info_js.get("root_dirname") or ""):
            info_js["local"] = True
            info_js["pkg_path"] = get_path_or_init(info_js)
        return info_js
    return line


def return_local_imps(file_path):
    local_imps = []
    dirname = os.path.dirname(file_path)
    imports_js = get_all_imports(file_path)
    for pkg, imps in imports_js.items():
        if pkg not in ['context', 'nulines']:
            full_imp_info = get_dot_fro_line(pkg, dirname, file_path=file_path, get_info=True)
            if full_imp_info.get("local"):
                local_imps.append(full_imp_info)
    return local_imps


def get_all_pkg_paths(file_path):
    pkg_paths = []
    local_imps = return_local_imps(file_path)
    for local_imp in local_imps:
        pkg_path = local_imp.get('pkg_path')
        if pkg_path:
            pkg_paths.append(pkg_path)
    return pkg_paths


# --- NEW: Build dependency graph and detect circular imports ---

def build_graph(main_directory):
    dirs, all_local_scripts = get_files_and_dirs(main_directory, allowd_exts='.py', files_only=True)
    graph = defaultdict(set)
    for file_path in all_local_scripts:
        deps = get_all_pkg_paths(file_path)
        for dep in deps:
            if dep:  # only valid files
                graph[file_path].add(dep)
    return graph


def find_cycles(graph):
    visited, stack, cycles = set(), [], []

    def dfs(node, path):
        visited.add(node)
        path.append(node)
        for dep in graph.get(node, []):
            if dep not in visited:
                dfs(dep, path.copy())
            elif dep in path:
                cycle_start = path.index(dep)
                cycles.append(path[cycle_start:] + [dep])

    for node in graph:
        if node not in visited:
            dfs(node, [])
    return cycles


if __name__ == "__main__":
    main_directory = "/home/flerb/Documents/pythonTools/modules/src/modules/abstract_utilities/src/abstract_utilities"
    graph = build_graph(main_directory)
    cycles = find_cycles(graph)

    if not cycles:
        print("✅ No circular imports found.")
    else:
        print("❌ Circular imports detected:")
        for cycle in cycles:
            print(" → ".join(cycle))



from ...class_utils import *
from ...compare_utils import *
from ...directory_utils import *
from ...imports import *
from ...list_utils import *
from ...path_utils import *
from ...safe_utils import *
from ...ssh_utils import *
from ...string_utils import *
from ...type_utils import *
from ...time_utils import *
from ...class_utils import *
from ...compare_utils import *
from ...directory_utils import *
from ...list_utils import *
from ...path_utils import *
from ...safe_utils import *
from ...ssh_utils import *
from ...string_utils import *
from ...type_utils import *
from ...time_utils import *
